//
//  BFCameraProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/17.
//

import Foundation
import BFCommonKit
import BFUIKit
import CoreMedia

class BFCameraProgressView: BFProgressBaseView {
    
    lazy var collectionV : UICollectionView = {
        let flowLayout = UICollectionViewFlowLayout()
        flowLayout.minimumLineSpacing = 0
        flowLayout.minimumInteritemSpacing = 0
        flowLayout.scrollDirection = .horizontal
        flowLayout.itemSize = CGSize(width: self.frame.width, height: self.frame.height)
        flowLayout.headerReferenceSize = CGSize(width: cScreenWidth / 2.0, height: 50)
        flowLayout.footerReferenceSize = CGSize(width: cScreenWidth / 2.0, height: 50)

        let cc = UICollectionView(frame: CGRect(x: 0, y: 0, width: self.width, height: self.height), collectionViewLayout: flowLayout)
        cc.delegate = self
        cc.dataSource = self
        cc.bounces = false
        cc.register(UICollectionReusableView.self, forSupplementaryViewOfKind: UICollectionView.elementKindSectionHeader, withReuseIdentifier:  String(describing: UICollectionReusableView.self))
        cc.register(UICollectionReusableView.self, forSupplementaryViewOfKind: UICollectionView.elementKindSectionFooter, withReuseIdentifier:  String(describing: UICollectionReusableView.self))
        cc.register(UICollectionViewCell.self, forCellWithReuseIdentifier: String(describing: UICollectionViewCell.self))
        cc.showsVerticalScrollIndicator = false
        cc.showsHorizontalScrollIndicator = false
//        cc.contentInset = UIEdgeInsets(top: 0, left: cScreenWidth / 2, bottom: 0, right: cScreenWidth / 2)
        cc.backgroundColor = UIColor.clear
        cc.decelerationRate = .fast
        
        return cc
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)

        addSubview(collectionV)
        insertSubview(collectionV, belowSubview: line)
        
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// 添加缩略图
    /// - Parameter images:
    func addThumbImages() {
        if let count = recordItem?.thumbImgs.count, count > 0 {
            DispatchQueue.main.async {
                self.collectionV.insertItems(at: [IndexPath(row: count - 1, section: 0)])
            }
        }
    }
    
    func updateCellWidth(index:Int, progress:CMTime){
        if let count = recordItem?.thumbImgs.count, count > 0 {
            collectionV.reloadItems(at: [IndexPath(row: count - 1, section: 0)])
            let p = CGPoint(x: progress.seconds * 70 / 5.0, y: 0)
            collectionV.contentOffset = p
        }else {
            collectionV.reloadData()
        }
    }
}

extension BFCameraProgressView: UICollectionViewDelegateFlowLayout, UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return recordItem?.thumbImgs.count ?? 0
    }
    
    func collectionView(_ collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, at indexPath: IndexPath) -> UICollectionReusableView {
        if kind == UICollectionView.elementKindSectionHeader || kind == UICollectionView.elementKindSectionFooter{
            let header = collectionView.dequeueReusableSupplementaryView(ofKind: kind, withReuseIdentifier: String(describing: UICollectionReusableView.self), for: indexPath)
            header.backgroundColor = .yellow
            return header
        }
        return UICollectionReusableView()
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        var size : CGSize = .zero
        if let img = recordItem?.thumbImgs[indexPath.row] {
            if let sticker = recordItem?.videoStickers.first(where: { m in
                m.thumImgs?.contains(img) ?? false
            }) {
                if img == sticker.thumImgs!.last {
                    size = CGSize(width: ((sticker.timelineOut - sticker.timelineIn) * 70.0 / 5.0).truncatingRemainder(dividingBy: 70.0), height: 50)
                    if (size.width < 0){
                        BFLog(1, message: "errror")
                        size = .zero
                    }
//                    BFLog(1, message: "currsize \(size), \(indexPath.row), \(sticker.timelineOut), \(sticker.timelineIn)")
                }else{
                    size = CGSize(width: 70.0, height: 50)
                }
            }
        }
        return size
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: String(describing: UICollectionViewCell.self), for: indexPath)
        cell.contentView.clipsToBounds = true
        var imgview : UIImageView? = cell.contentView.viewWithTag(13000) as? UIImageView
        if  imgview == nil {
            imgview = UIImageView(frame: CGRect(x: 0, y: 0, width: 70, height: 50))
            imgview?.contentMode = .scaleAspectFill
            imgview?.tag = 13000
            cell.contentView.addSubview(imgview!)
        }
        
        imgview?.image = recordItem?.thumbImgs[indexPath.row]
        
        return cell
    }
}

extension BFCameraProgressView {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if isDrag {
            let totalW = (scrollView.contentOffset.x + cScreenWidth / 2.0) / scrollView.contentSize.width
            
            dragScrollProgressHandle?(false,Float(min(1, max(0, totalW))))
        }
    }

    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        let totalW = (scrollView.contentOffset.x + cScreenWidth / 2.0) / scrollView.contentSize.width
        dragStartHandle?()
        
        dragScrollProgressHandle?(true,Float(min(1, max(0, totalW))))
    }

    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            let totalW = (scrollView.contentOffset.x + cScreenWidth / 2.0) / scrollView.contentSize.width
            isDrag = false
            dragEndHandle?(Float(min(1, max(0, totalW))))
        }
    }

    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        let totalW = (scrollView.contentOffset.x + cScreenWidth / 2.0) / scrollView.contentSize.width
        isDrag = false
        dragEndHandle?(Float(min(1, max(0, totalW))))
    }
}
